import datetime
from django.utils.text import slugify


def get_file_path(instance, filename):
    """
    Формирует путь файла относительно года и месяца, чтобы множество файлов не скапливались на одном уровне.
    """
    import uuid
    ext = filename.split('.')[-1]
    fname = slugify(".".join(filename.split('.')[:-1]))
    if len(fname) < 3:
        fname += str(uuid.uuid4())
    today = datetime.date.today()
    filename = f'{fname}.{ext}'
    return f'uploads/{today.year}/{today.month}/{filename}'



def check_redirect(request, context):
    """
    if 'redirect' key in context dict than redirects to url ('redirect':value).
    request is need to exclude recursive redirect
    :param request:
    :param context:
    :return: httpresponse : redirects to given url
    """
    redirect_context = context.get('redirect', None)
    if redirect_context and request.path != redirect_context:
        return context['redirect']



from django.contrib import admin
from django import forms
from ..models.base_page import BasePage
from modeltranslation.admin import TabbedTranslationAdmin
from polymorphic_tree.admin import PolymorphicMPTTParentModelAdmin, PolymorphicMPTTChildModelAdmin, PolymorpicMPTTAdminForm
from ..utils.get_garpix_page_models import get_garpix_page_models


class BasePageAdmin(TabbedTranslationAdmin, PolymorphicMPTTChildModelAdmin):
    base_model = BasePage

    empty_value_display = '- нет -'
    save_on_top = True
    view_on_site = True

    date_hierarchy = 'created_at'
    prepopulated_fields = {'slug': ('title',)}

    search_fields = ('title',)
    list_filter = ('is_active', 'created_at', 'updated_at')
    actions = ('clone_object', 'rebuild')

    list_display = ('title', 'created_at', 'is_active', 'get_absolute_url',)
    list_editable = ('is_active',)

    readonly_fields = ('created_at', 'updated_at')

    def get_form(self, request, *args, **kwargs):
        form = super().get_form(request, *args, **kwargs)
        # form.current_user = request.user
        return form


@admin.register(BasePage)
class RealBasePageAdmin(TabbedTranslationAdmin, PolymorphicMPTTParentModelAdmin):
    """
    Стандартные настройки для базовых страниц.
    """
    base_model = BasePage
    child_models = get_garpix_page_models()

    empty_value_display = '- нет -'
    save_on_top = True
    view_on_site = True

    date_hierarchy = 'created_at'
    prepopulated_fields = {'slug': ('title',)}

    search_fields = ('title',)
    list_filter = ('is_active', 'created_at', 'updated_at')
    actions = ('clone_object', 'rebuild')

    list_display = ('title', 'created_at', 'is_active', 'get_absolute_url',)
    list_editable = ('is_active',)

    readonly_fields = ('created_at', 'updated_at')

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if 'initial' in kwargs:
            self.fields['parent'].queryset = self.model.get_available_page_parents()
            self.fields['parent'].default = self.model.get_default_parent()

    def clone_object(self, request, queryset):
        """Копирование(клонирование) выбранных объектов - action"""
        for obj in queryset:
            prefix = '-CLONE'
            clone = obj
            clone.title += prefix
            clone.slug += prefix
            clone.id = None
            clone.is_active = False
            clone.save()

    clone_object.short_description = 'Клонировать объект'

    def _rebuild(self):
        try:
            self.model.objects.rebuild()
        except:  # noqa
            print('[ERROR]: Ошибка при перезагрузки древовидной структуры')

    def rebuild(self, request, queryset):
        """Пересорбать МПТТ модель. Иногда требуется для перезагрузки дерева."""
        self._rebuild()

    rebuild.short_description = 'Пересобрать пункты раздела'

    def save_model(self, request, obj, form, change):
        self._rebuild()
        super().save_model(request, obj, form, change)



import django.apps
from ..models import BasePage


model_list = []
for model in django.apps.apps.get_models():
    try:
        if model.is_for_page_view() and model is not BasePage:
            model_list.append(model)
    except:  # noqa
        pass


def get_garpix_page_models():
    return model_list



def check_sites(cleaned_data):
    """
    returns True if sites params same as parent or children objects (instance have same sites id as children and parent)
     or if site params of instance have no sense (instance, parent or children have no sites params,)

    return False if site params have not intersection with parent/children of instance (did not have SAME site id)
    :return: bool
    """
    result = True
    if cleaned_data.get('sites', None):
        instance_sites = set(cleaned_data['sites'].all().values_list('id', flat=True))
        if cleaned_data.get('parent', None) and cleaned_data['parent'] is not None \
           and hasattr(cleaned_data['parent'], 'sites') and cleaned_data['parent'].sites is not None:
            parent_sites = set(cleaned_data['parent'].sites.all().values_list('id', flat=True))
            result = bool(instance_sites.intersection(parent_sites))
    return result



from modeltranslation.translator import TranslationOptions, register
from ..models import BasePage


@register(BasePage)
class BasePageTranslationOptions(TranslationOptions):
    fields = ('title', 'seo_title', 'seo_keywords', 'seo_description', 'seo_author')



from django.conf import settings
from django.http import Http404
from ..utils.get_garpix_page_models import get_garpix_page_models
from django.utils import translation
from ..utils.check_redirect import check_redirect
from django.shortcuts import redirect
from django.views.generic import DetailView


class PageView(DetailView):
    def get_template_names(self):
        """
        Метод для получения темплейта для страницы.
        Структуру темплейтов можно посмотреть в модели page.models.Page.
        """
        if self.object is not None:
            return [self.object.get_template()]
        return []

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        if self.object is not None:
            context.update(self.object.get_context(self.request, object=self.object))
        return context

    def _get_home_page(self):
        home_pages = []
        for Model in get_garpix_page_models():
            home_page = Model.on_site.filter(slug='', is_active=True).first()
            if home_pages is not None:
                return home_page
        return Http404

    def _get_current_language_code_url_prefix(self):
        current_language_code_url_prefix = translation.get_language()
        try:
            use_default_prefix = settings.USE_DEFAULT_LANGUAGE_PREFIX
        except:  # noqa
            use_default_prefix = True
        if not use_default_prefix and current_language_code_url_prefix == settings.LANGUAGE_CODE:
            current_language_code_url_prefix = ''
        elif current_language_code_url_prefix is None:
            current_language_code_url_prefix = ''
        else:
            current_language_code_url_prefix = '/' + current_language_code_url_prefix
        return current_language_code_url_prefix

    def _get_object_list_by_url(self, url):
        obj_list = []
        slugs = url.rstrip('/').split('/')
        slug = slugs[-1]
        parent_slug = slugs[-2] if len(slugs) > 2 else None
        for model in get_garpix_page_models():
            if parent_slug:
                _obj = model.on_site.filter(slug=slug, parent__slug=parent_slug).first()
            else:
                _obj = model.on_site.filter(slug=slug).first()
            if _obj:
                obj_list.append(_obj)
        return obj_list

    def get_object(self, queryset=None):
        """
        Метод для получения объекта страницы.
        Сравнивает текущий урл со slug, которые мы получаем из родительских страниц нашей страницы.
        """
        current_language_code_url_prefix = self._get_current_language_code_url_prefix()
        url = self.kwargs.get('url', None)
        # home pages
        if url is None or url == '':
            return self._get_home_page()
        # get page object
        obj_list = self._get_object_list_by_url(url)
        obj = None
        for item in obj_list:
            if item.get_absolute_url() == f'{current_language_code_url_prefix}/{url}':
                obj = item
                break
        if not obj:
            raise Http404
        if not obj.is_active:
            raise Http404
        return obj

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        if getattr(self.object, 'login_required', None):
            if not self.request.user.is_authenticated:
                return redirect(settings.LOGIN_URL)
        context = self.get_context_data(object=self.object)
        redir = check_redirect(request, context)
        if redir:
            return redirect(redir)
        return self.render_to_response(context)

    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        if getattr(self.object, 'login_required', None):
            if not self.request.user.is_authenticated:
                return redirect(settings.LOGIN_URL)
        context = self.get_context_data(object=self.object)
        redir = check_redirect(request, context)
        if redir:
            return redirect(redir)
        return self.render_to_response(context)



from django.utils.functional import cached_property
from django.db import models
from django.utils import translation
from django.conf import settings
from django.utils.module_loading import import_string
from django.contrib.sites.models import Site
from django.contrib.sites.managers import CurrentSiteManager
from django.utils.functional import lazy
from garpix_page.utils.get_file_path import get_file_path
from polymorphic_tree.models import PolymorphicMPTTModel, PolymorphicTreeForeignKey, PolymorphicMPTTModelManager


class GCurrentSiteManager(CurrentSiteManager):
    use_in_migrations = False


class BasePage(PolymorphicMPTTModel):
    """
    Базовая страница, на основе которой создаются все прочие страницы.
    """
    title = models.CharField(max_length=255, verbose_name='Название')
    is_active = models.BooleanField(default=True, verbose_name='Включено')
    display_on_sitemap = models.BooleanField(default=True, verbose_name='Отображать в карте сайта')
    slug = models.SlugField(max_length=150, verbose_name='ЧПУ', blank=True, default='')
    sites = models.ManyToManyField(Site, verbose_name='Сайты для отображения')
    created_at = models.DateTimeField(auto_now_add=True, verbose_name='Дата создания')
    updated_at = models.DateTimeField(auto_now=True, verbose_name='Дата изменения')
    seo_title = models.CharField(max_length=250, verbose_name='SEO заголовок страницы (title)', blank=True, default='')
    seo_keywords = models.CharField(max_length=250, verbose_name='SEO ключевые слова (keywords)', blank=True,
                                    default='')
    seo_description = models.TextField(verbose_name='SEO описание (description)', blank=True, default='')
    seo_author = models.CharField(max_length=250, verbose_name='SEO автор (author)', blank=True, default='')
    seo_og_type = models.CharField(max_length=250, verbose_name='SEO og:type', blank=True, default="website")
    seo_image = models.FileField(upload_to=get_file_path, blank=True, null=True, verbose_name='SEO изображение')
    parent = PolymorphicTreeForeignKey('self', null=True, blank=True, related_name='children',
                                       db_index=True, verbose_name='Родительская страница', on_delete=models.SET_NULL,
                                       limit_choices_to={})

    # objects = models.Manager()
    objects = PolymorphicMPTTModelManager()
    on_site = GCurrentSiteManager()

    template = 'garpix_page/default.html'

    @classmethod
    def get_page_types(cls):
        return cls.page_types

    @classmethod
    def get_choices_page_types(cls):
        new_dict = {k: settings.PAGE_TYPES[k] for k in cls.get_page_types()}
        print('new_dict', new_dict)
        choices = [(k, v['title']) for k, v in new_dict.items()]
        return choices

    # def __init__(self, *args, **kwargs):
    #     super().__init__(*args, **kwargs)
    #     self._meta.get_field('page_type').choices = lazy(self._get_choices_page_types(), list)()

    # end available page types

    class Meta(PolymorphicMPTTModel.Meta):
        verbose_name = 'Структура страниц'
        verbose_name_plural = 'Структура страниц'
        ordering = ('created_at', 'title',)

    def __str__(self):
        return self.title

    def get_verbose_model_name(self):
        return self._meta.verbose_name

    get_verbose_model_name.short_description = 'Название модели'

    def get_absolute_url(self):
        return self.absolute_url

    get_absolute_url.short_description = 'URL'

    @cached_property
    def absolute_url(self):
        current_language_code_url_prefix = translation.get_language()
        try:
            use_default_prefix = settings.USE_DEFAULT_LANGUAGE_PREFIX
        except:  # noqa
            use_default_prefix = True
        if not use_default_prefix and current_language_code_url_prefix == settings.LANGUAGE_CODE:
            current_language_code_url_prefix = ''
        elif current_language_code_url_prefix is None:
            current_language_code_url_prefix = ''
        else:
            current_language_code_url_prefix = '/' + current_language_code_url_prefix

        if self.slug:
            obj = self
            url_arr = [self.slug]
            while obj.parent is not None:
                obj = obj.parent
                if obj.slug:
                    url_arr.insert(0, obj.slug)
            return "{}/{}".format(current_language_code_url_prefix, '/'.join(url_arr))
        return "{}".format(current_language_code_url_prefix) if len(current_language_code_url_prefix) > 1 else '/'

    absolute_url.short_description = 'URL'

    @cached_property
    def get_sites(self):
        res = 'n/a'
        if self.sites.all().count() > 0:
            res = ''
            for site in self.sites.all():
                res += f'{site.domain} '
        return res

    get_sites.short_description = 'Sites'

    def get_template(self):
        return self.template

    def get_context(self, request=None, *args, **kwargs):
        context = {
            'request': request,
            'object': self,
        }
        return context

    @classmethod
    def is_for_page_view(cls):
        return True

    def get_breadcrumbs(self):
        result = []
        obj = self
        result.append(obj)
        while obj.parent is not None:
            result.insert(0, obj.parent)
            obj = obj.parent
        return result



from rest_framework import status
from django.utils.translation import activate
from rest_framework import views
from rest_framework.response import Response
from django.contrib.auth import get_user_model
import django.apps
from django.conf import settings
from django.utils.module_loading import import_string


model_list = []
for model in django.apps.apps.get_models():
    try:
        if model.is_for_page_view():
            model_list.append(model)
    except:  # noqa
        pass


class PageApiView(views.APIView):

    @staticmethod
    def get_common_context(request=None, *args, **kwargs):
        if not hasattr(settings, 'COMMON_CONTEXT'):
            return {}
        context_function = import_string(settings.COMMON_CONTEXT)
        return context_function(request, *args, **kwargs)

    @staticmethod
    def get_instance_by_slug(slug, model_list):
        for m in model_list:
            instance = m.objects.filter(slug=slug).first()
            if instance:
                return instance
        return None

    def get_object(self, slug):
        obj = self.get_instance_by_slug(slug, model_list)
        if obj:
            return obj
        else:
            return None

    def get(self, request, slugs):
        language = "ru"
        if 'HTTP_ACCEPT_LANGUAGE' in request.META:
            language = request.META['HTTP_ACCEPT_LANGUAGE']
        activate(language)

        slug_list = slugs.split('/')
        slug = slug_list.pop(-1)
        page = self.get_object(slug)

        if request.user.is_authenticated:
            user = get_user_model().objects.get(pk=request.user.pk)
        else:
            user = None

        common_context = self.get_common_context(request, obj=page, user=user)

        if page is None:
            data = {
                'type': None,
                'init_state': common_context
            }
            return Response(data, status=status.HTTP_404_NOT_FOUND)

        page_context = page.get_context(request, obj=page, user=user)
        common_context.update(page_context)

        data = {
            'type': None,
            'init_state': common_context,
        }
        return Response(data)
